;********************* Tiny13V Sanduhr-Ersatz **********************
;
;In diesem Projekt versuche ich alle (mir bekannten) Mglichkeiten
;zur Verringerung der Stromaufnahme zu zeigen.
;
;Da batteriebetrieben,sollte ein Tiny13"V" eingesetzt werden,
;Der Tiny13V hat als untere Grenze der Versorgungsspannung
;laut Datenblatt 1,8V hat.Mein Tiny13V funktioniert sogar bis
;1,1V. --> MUSS NICHT MIT JEDEM Tiny13V SO SEIN! <--
;
;Zur Not kann aber auch ein "normaler" Tiny13 mit 3 Stk. Knopfzellen
;zu je 1,5V versorgt werden.Das Problem dabei ist nur, da man fr
;diese Knopfzellen keinen geeigneten Halter im Handel findet.
;Heisser Tip zum Halter : Es gibt Feuerzeuge, die eine LED als
;"Taschenlampe" am unteren Ende haben.Hier kann man einen geeigneten
;Halter fr Knopfzellen ausbauen.
;
;Fuse Settings:
;**************
;Clock = 600KHz (4,8MHz/8),Brown-Out disabled -> weniger Ruhestrom!
;SUT1..0 = 00 (6Clk+14Clk) = Start-Up erfolgt schnell -> weniger
;Ruhezyklen = daher im Schnitt weniger Stromverbrauch.
;
;Es wrde noch die Mglichkeit geben, den Tiny13V mit 128KHz
;zu takten (Watchdog-Clock).Dies gbe zustzlich eine Verringerung
;der Stromaufnahme.Allerdings knnen dann Schwierigkeiten beim
;programmieren ohne STK500 auftreten.Deswegen habe ich diese
;Mglichkeit nicht genutzt um alle gngigen ISP-Programmer verwenden zu
;knnen.
;
;1 min nach Start ertnt Signal (3 x Beep)
;2 min nach Start ertnt Signalfolge mit 5 absteigenden Beep-
;Frequenzen und Programm stoppt.
;Programmablauf kann jederzeit mit Taste"Stop" beendet werden.
;
;Taste "Start" an RESET = PB5 (Pullup mit 50K-100k an Vcc)
;Taste "Stop" an INT0 = PB1 (eventuell mit 50k-100k Pullup an Vcc)
;
;Bei drcken der Tasten "Start" oder "Stop" erfolgt ein Beep
;als Rckmeldung.
;
;Zur Kontrolle blinkt LED (Kathode ber Rv an PB0,Anode an Vcc)
;im Sekundenrhythmus.Rv ist bei mir 560 Ohm (bei 3V Vcc).
;Die LED dient dabei gleichzeitig auch als Spannungskontrolle.
;Von 5V bis ca. 2V ist das blinken noch gut zu sehen, ab 
;1,9V dann fast nicht mehr (Durchfluss-Spannung einer roten
;LED ist 1,8V!)
;
;Piezo-Buzzer (ohne eingebauter Elektronik) ber 1k5 an PB2 und GND.
;1k5 nur wegen eventueller Umprogrammierung.Ohne 1k5 wird Programmiersignal
;"SCK" zu stark belastet und eine Programmierung ist eventuell nicht
;mehr mglich.Nach der Programmierung kann 1k5 auch entfallen (Brcke).
;Mit 1k5 ist die Lautstrke des Piezo-Buzzers allerdings geringer.
;
;Stromaufnahme aktiv bei 3V (z.B. Knopfzelle) = ca. 80 uA
;                    bei 5V (z.B. Goldcap)    = ca. 200uA
;Stromaufnahme im Stop-Modus                  = ca. 1,2uA
;
; -->    OSCCAL-Wert an den jeweiligen Tiny13V anpassen       <--
; -->    und/oder den Timer0reload-Wert ndern um halbwegs    <--
; -->    genaue Sekundenintervalle zu erhalten.               <--
;
;********************************************************************

;Define Registers
;-----------------
.def Ticks		= r17	;counts TOV0-Interrupts.5 Ticks=0,998 seconds
.def Second		= r18
.def Minute		= r19
.def Work1		= r20	;Working registers
.def Work2		= r21
.def Work3		= r22
.def Temp1		= r1	;Temporary Registers
.def Temp2		= r2
.def Temp3		= r3
.def Temp4		= r4
.def Save_SREG	= r5

; Define Pins:
;-------------
.equ	led_pin		= 0		;Heartbeat LED on PB0
.equ 	Buzzer 		= 2		;Piezo-Buzzer on PB2
.equ 	Stop_Taste	= 1		;Stop-Switch on PB1

;Define Constants:
;----------------- 
.equ Timer0reload = 117		;Timer Reload Value for 199,678msec TOV0-Interval
							;This is a calculated Value. Real Value depends
							;on the accuracy of the internal Oscillator ! 
														
.nolist
.include "tn13def.inc"
.list
	.cseg
	.org	0x0000

;Interrupt Jump Table
;--------------------
	rjmp	Reset
	rjmp	Int0_ ; jump to INT0 Routine
	reti
	rjmp	TOV0_ ; jump to Timer0 Overflow
	reti
	reti
	reti
	reti
	reti
	reti

Reset:

;*************************************
;  Initialize Tiny13
;*************************************

;	ldi Work1,0x67 			;OSCCAL 4,8MHz for --> MY Tiny13 <-- only
;	out OSCCAL,Work1
	ldi	Work1,RamEnd		;init Stack
	out	SPL,Work1

;Disable Watchdog to save Power
;------------------------------
	wdr
	in	Work1,MCUSR
	cbr	Work1,1<<WDRF 			;clear WDRF
	out	MCUSR,Work1
	ori	Work1,(1<<WDCE)|(1<<WDE)	;clear WDCE and WDE of Watchdog
	out	WDTCR,Work1
;now turn off Watchdog
	cbr	Work1,1<<WDE				;clear WDE	
	out	WDTCR,Work1


; configure Output/Input
;-----------------------
	sbi	DDRB,led_pin		;PB0 as Output for Heartbeat LED on PB0
	sbi	DDRB,Buzzer			;PB2 as Output for Piezo-Buzzer
	cbi	DDRB,Stop_Taste		;PB1 as Input for INT0 (Stop-Button)
	sbi	PORTB,Stop_Taste	;Enable internal Pullup for PB1
	sbi	PORTB,led_pin		;LED off
	cbi DDRB,3				;config unused PB3/PB4 as Input
	cbi DDRB,4
	sbi PORTB,3				;Enable internal Pullups for PB3/PB4
	sbi PORTB,4

; configure INT0 
;-----------------
	in	Work1,MCUCR
	andi	Work1,0xFC 		;clear ISC01,ISC00 only
	ori	Work1,1<<ISC01 		;set ISC01 only-> falling Edge of INT0
					 		;generates Interrupt
	out	MCUCR,Work1

; configure Timer0
;-----------------
	in Work1,TCCR0B
	ldi	Work1,(1<<CS02)|(1<<CS00);	;Clock/1024
	cbr Work1,1<<WGM02				;Normal Counter Mode,
	out	TCCR0B,Work1
	clr	Work1							;Timer0 as Counter
	out	TCCR0A,Work1
;calculate TCNT0-Value:256-Timer0reload = counts to Overflow
	clr	Work1				
	subi Work1,Timer0reload 
	out	TCNT0,Work1
	in	Work1,TIMSK0
	ori	Work1,1<<TOIE0	;set TOIE0 -> Counter0 Overflow Interrupt enable
	out	TIMSK0,Work1
	in	Work1,GIMSK	
	ori	Work1,1<<INT0		;set INT0 -> enable Interrupt Flag 0
	out	GIMSK,Work1

; shut down ADC /Analog Comparator to save power
;-----------------------------------------------
	sbi	ACSR,ACD		;disable Analog Comparator
	cbi	ADCSRA,ADEN		;disable ADC

;Clear Variables.Must be cleared,otherwise content
;after Power-Up or Reset is random!
;---------------
	clr	Ticks
	clr Second
	clr Minute

	sei				;enable global Interrupts

;*************************************
;  End of Initialize
;*************************************

;Sound Buzzer -> Start-Signal
;----------------------------

	ldi	Work3,0x40		;set Buzzer Pitch
	ldi	Work1,0xFF		;set Buzzer Duration	
	rcall L1000			;Start Buzzer

;*****************************
; Main-Loop
;*****************************

L0001:
	cpi	Ticks,5			;5 Ticks (= 1 second) gone?
	breq	L0002		;yes
	rjmp	L0003		;no

L0002:
	clr	Ticks			;reset ticks to zero
;flash LED every second
	cbi	PORTB,led_pin	;LED on
	ldi	Work1,0x0A		;0x0A = approx. 7,5msec LED on
	rcall	L2000	    ;delay
	sbi	PORTB,led_pin   ;LED off
	subi Second,-1 		;increment Second
L0003:
	cpi	Second,60		;60 seconds gone?
	breq	L0004		;yes
	rjmp	L0005 		;no,go to sleep (Idle Mode)

L0004:
	clr	Second
	subi Minute,-1 		;increment Minute
	cpi	Minute,2 		;2 Minutes gone?
	breq	L0010		;yes,Time to Shut Down

;********************************
; 1 Minute gone : Beep 3 times
;********************************
	ldi	Work3,0x40		;set Pitch
	ldi	Work1,0xff		;and Duration for the Beeps
	mov Temp3,Work1		;save Duration

;first Beep
	rcall L1000
	
;some delay until next Beep
	ldi	Work1,0x64
	rcall	L2000 		;Delay
	mov Work1,Temp3		;restore Duration

;second Beep
	rcall L1000		

;some delay until next Beep
	ldi	Work1,0x64
	rcall	L2000		;Delay
	mov Work1,Temp3		;restore Duration

;third Beep
	rcall L1000

;Go to sleep (Idle-Mode),wait for Timer-Interrupt
L0005:
	in	Work1,MCUCR
	andi	Work1,0x07	;clear SM0,SM1,SE
	ori	Work1,1<<SE		;set Sleep enable
	out	MCUCR,Work1
	sleep				;wait for Timer-Interrupt
	rjmp	L0001		;go to Main-Loop 


;******************************
;	2 Minutes gone : Finish !
;     Play "Finish"-Melody
; Register "Ticks" is used here
; because it is not used anymore
;******************************

L0010:
	cli
	clr	Work1
	out	TCCR0B,Work1 	;Stop Timer
;-----------------------------
;Sound Buzzer with five different Beeps
;First Beep has highest,last Beep has lowest Pitch  
;-----------------------------
	clr	Ticks			;clear Loop-Counter for 5 Beeps
;Load Values for first (highest) Beep
	ldi	Work3,0x30		;set Pitch
	ldi	Work1,0xE0		;and Duration for the Beeps
	mov Temp3,Work1		;save Duration

L0011:
; Beep 5 times
	rcall L1000
	
;some delay until next Beep
	ldi	Work1,0x00
	rcall	L2000 		;Delay
	mov Work1,Temp3		;restore Duration

;five Beeps done?
	cpi	Ticks,4			;Ticks = 4?
	breq L0104			;yes,go to sleep

; Work3 = Work3 + 10 -> beep next lower pitch
;---------------------
	subi Work3,-10
	subi Ticks,-1		;increment Loop-Counter
	brcc L0104			;Loop-Counter zero?
	rjmp	L0011		;no

;*********************************************
; Timer0 Overflow goes here.Increment Ticks
;*********************************************

TOV0_:
	in Save_SREG,SREG
	mov Temp4,Work1
;according to the Datasheet,it is recommended to clear SE
;after leaving Sleep-Mode
	in	Work1,MCUCR
	andi	r23,0xDF 	;clear SE only
	out	MCUCR,Work1
;-----------------------
;calculate TCNT0 reload Value:256-Timer0reload
	clr	Work1				
	subi Work1,Timer0reload
;end of calc
	out	TCNT0,Work1
	subi	Ticks,-1 	;increment Ticks
	mov Work1,Temp4
	out SREG,Save_SREG
	reti

;*********************************************
; INT0 goes here -> Stop Program and sleep
; in Power-Down Mode.Wake up on Reset only
;*********************************************

Int0_:
	cli						;disable all Interrupts

;Sound Buzzer -> Stop-Signal
	ldi Work3,0x28 		;set Buzzer Pitch 
	ldi Work1,0xFF		;and Duration

;sound Buzzer
	rcall L1000

L0104:
	clr	Work1				;stop Timer0
	out	TCCR0B,Work1
	sbi	PORTB,led_pin 		;LED off
;now activate Power down
	in	Work1,MCUCR
	andi	Work1,0xE7	;clear SM0
	ori	Work1,0x30		;set SE,SM1.Sleep-Mode= Power down
	out	MCUCR,Work1
L0105:
	sleep				;wait for Button "Start" = Reset
	rjmp	L0105		;this loop should never been executed!

;***********************************
;         Subroutines
;***********************************


;----------------
;activate Beeper.
;----------------

;Beep-Duration in Work1
L1000:
	subi Work1,1
	sbi	PORTB,Buzzer
	rcall	L1001		;Pitch-Delay
	cbi	PORTB,Buzzer
	rcall	L1001		;Pitch-Delay
	tst Work1
	brne	L1000		;not finished jet
	ret					;Beep done

L1001:
;Beep-Pitch in Work3
	mov	Work2,Work3
L1002:
	subi Work2,1
	tst Work2
	brne	L1002
	ret

;-------------------
;End activate Beeper
;-------------------

;--------------------
;Multi-purpose Delay
;--------------------
L2000:
;delay depends on Work1 times Work3
	mov Temp1,Work3 	
	mov Temp2,Work1
	tst Work1
	breq	L2003	;exit delay
L2001:
	ldi	Work3,0x96
L2002:
	subi Work3,1
	brne	L2002
	subi Work1,1
	brne	L2001
L2003:
	mov Work1,Temp2	
	mov Work3,Temp1
	ret

;--------------------
;End of Multi-purpose Delay
;--------------------
